Function streamingServer_Initialize(msgPort As Object, userVariables As Object, bsp as Object)
	print "****************************************************************************"
	print "Streaming Server Control Plugin - Start"
 	print "****************************************************************************"

	ssm = newSSControlManager(msgPort, userVariables, bsp)

	return ssm
End Function

Function newSSControlManager(mp as Object, userVariables As Object, bsp as Object)

	print "=== newSSControlManager start"

	ssm = {}

	ssm.version = "0.0.5"
	''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	' DEBUG: set this to "1"
	ssm.debugMode$ = "1"
	if userVariables.ssDebugMode <> invalid and userVariables.ssDebugMode.defaultValue$ <> invalid then
		ssm.debugMode$ = userVariables.ssDebugMode.defaultValue$
	end if
	print "=== ssm.debugMode$: ";ssm.debugMode$
	''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	ssm.PrDebug = PrDebug
	ssm.PrDebug("ssm.version = "+ssm.version)

	ssm.onScreenMessageTime = 10000
	ssm.msgPort = mp
	ssm.userVariables = userVariables
	ssm.bsp = bsp

	ssm.networkConfig = CreateObject("roNetworkConfiguration", 0)
	ssm.deviceInfo = CreateObject("roDeviceInfo")
	ssm.hp = CreateObject("roNetworkHotplug")
	ssm.hp.setPort(mp)

	ssm.ProcessEvent = ssm_ProcessEvent
	ssm.AddHttpHandlers = ssm_AddHttpHandlers
	ssm.StartStreaming = ssm_StartStreaming
	ssm.StopStreaming = ssm_StopStreaming
	ssm.SendPluginEvent = ssm_SendPluginEvent

	' User settings are stored in the registry
	ssm.registrySection = CreateObject("roRegistrySection", "ssm")
	ssm.InitStoredConfig = ssm_InitStoredConfig 
	ssm.ReadValueFromRegistry = ssm_ReadValueFromRegistry 
	ssm.GetValue = ssm_GetValue 
	ssm.SaveValue = ssm_SaveValue 
	ssm.InitStoredConfig()

	ssm_streamCacheDir$ = "sd:/streamCache"
	CreateDirectory(ssm_streamCacheDir$)

	' set monitor timer to finish init for things that cannot be done here
	ssm.monitorTimer = CreateObject("roTimer")
	ssm.monitorTimer.SetPort(ssm.msgPort)
	ssm.monitorTimer.SetElapsed(1, 0)
	ssm.monitorTimer.Start()

	ssm.GetJob = ssm_GetJob
	ssm.SaveJob = ssm_SaveJob
	ssm.streamer = invalid
	ssm.streamingStatus = "Stopped"
	ssm.pipeline$ = ""
	ssm.lastError$ = ""
	ssm.currentStreamingJob = ssm.GetJob()

	ssm.GetStreamingStatus = GetStreamingStatus

	ssm.SendHttpResponse = SendHttpResponse
	ssm.Notify = Notify
	ssm.ShowLoadingMessage = ShowLoadingMessage

	ssm.InitLwsCredentials = InitLwsCredentials
	ssm.AddUrls = AddUrls 

	ssm.AddHttpHandlers()
	
print "=== 1"
	ssm.PrDebug("Should we start streaming? "+ssm.storedConfig["action"].value)
	if ssm.storedConfig["action"].value = "play" then
		ssm.StartStreaming()
	end if

	return ssm
End Function

Function ssm_ProcessEvent(event As Object) as Boolean
	retval = false
	if type(event) = "roAssociativeArray" then
		if type(event["EventType"]) = "roString" then
			m.PrDebug("Received event type: "+event["EventType"])
			if (event["EventType"] = "SEND_PLUGIN_MESSAGE") then
				if event["PluginName"] = "streamingServer" then
					pluginMessage$ = event["PluginMessage"]
					m.PrDebug("SEND_PLUGIN_MESSAGE:"+pluginMessage$)
					if pluginMessage$ = "startStreaming" then
						m.StartStreaming()
					else if pluginMessage$ = "stopStreaming" then
						m.StopStreaming()
					endif
					retval = true
				end if
			endif
		endif
	else if type(event) = "roTimerEvent" then
		if type(m.monitorTimer) = "roTimer" and event.GetSourceIdentity() = m.monitorTimer.GetIdentity() then
			retval = true
		end if
	else if type(event) = "roMediaStreamerEvent" then
		eventType = event.GetEvent()
		if eventType = 0 then
			m.PrDebug("Streaming stopped")
			m.streamingStatus = "Stopped"
		else
			m.streamingStatus = "Error"
			m.lastError$ = m.streamer.GetFailureReason()
			m.PrDebug("!!!--- Streaming error: "+m.lastError$)
		end if
		retval = true
	Else If type(event) = "roNetworkAttached" Then
		m.PrDebug("Received roNetworkAttached for nic " + str(event.GetInt()))
		' Force network config to be read again
		m.receivedIpAddr  = false
		' Wait till we get an actual IP address
		while not m.receivedIpAddr
			If type(m.networkConfig) = "roNetworkConfiguration" Then
				cc = m.networkConfig.GetCurrentConfig()
				If type(cc) = "roAssociativeArray" Then
					If cc.ip4_address <> "" Then
						m.receivedIpAddr = true
						m.PrDebug("Received IP addr: " + cc.ip4_address)
						if m.storedConfig["action"].value = "play" then
							m.StartStreaming()
						Else
							m.ShowLoadingMessage(true)
						End If
					End If
				End If
			End If
		End while
		retval = true
	Else If type(event) = "roNetworkDetached" Then
		m.StopStreaming()
		m.Notify("Network connection lost for interface " + str(event.GetInt()), "", m.onScreenMessageTime)
		If type(m.networkConfig) <> "roAssociativeArray" Then
			m.receivedIpAddr  = false
			m.Notify("Network connection lost", "No network connection", -1)
		End If
		retval = true
	Endif
	return retval
End Function

Sub ssm_AddHttpHandlers()

	m.PrDebug("ssm_AddHttpHandlers start")

	m.InitLwsCredentials()

	getDeviceInfoAA = { HandleEvent: GetDeviceInfo, mVar: m }
	m.bsp.sign.localServer.AddGetFromEvent({ url_path: "/GetDeviceInfo", user_data: getDeviceInfoAA, passwords: m.credentials })
	getNetworkInfoAA = { HandleEvent: GetNetworkInfo, mVar: m }
	m.bsp.sign.localServer.AddGetFromEvent({ url_path: "/GetNetworkInfo", user_data: getNetworkInfoAA, passwords: m.credentials })
	setNetworkSettingsJobAA = { HandleEvent: SetNetworkSettings, mVar: m }
	m.bsp.sign.localServer.AddPostToString({ url_path: "/SetNetworkSettings", user_data: setNetworkSettingsJobAA, passwords: m.credentials })
	getFolderContentsAA = { HandleEvent: GetFolderContents, mVar: m }
	m.bsp.sign.localServer.AddGetFromEvent({ url_path: "/GetFolderContents", user_data: getFolderContentsAA, passwords: m.credentials })
	getStreamingStatusAA = { HandleEvent: GetStreamingStatus, mVar: m }
	m.bsp.sign.localServer.AddGetFromEvent({ url_path: "/GetStreamingStatus", user_data: getStreamingStatusAA, passwords: m.credentials })
	getCurrentStreamingJobAA = { HandleEvent: GetCurrentStreamingJob, mVar: m }
	m.bsp.sign.localServer.AddGetFromEvent({ url_path: "/GetCurrentStreamingJob", user_data: getCurrentStreamingJobAA, passwords: m.credentials })
	setStreamingJobAA = { HandleEvent: SetStreamingJob, mVar: m }
	m.bsp.sign.localServer.AddPostToString({ url_path: "/SetStreamingJob", user_data: setStreamingJobAA, passwords: m.credentials })
	getLwsCredentialsAA = { HandleEvent: GetLwsCredentials, mVar: m }
	m.bsp.sign.localServer.AddGetFromEvent({ url_path: "/GetLwsCredentials", user_data: getLwsCredentialsAA, passwords: m.credentials })
	saveLwsCredentialsAA = { HandleEvent: SaveLwsCredentials, mVar: m }
	m.bsp.sign.localServer.AddPostToString({ url_path: "/SaveLwsCredentials", user_data: saveLwsCredentialsAA, passwords: m.credentials })

	' All dependent assets, like javascript files, css, ...
	m.AddUrls("", "web-client")
print "=== credentials=";m.credentials

	m.ShowLoadingMessage(true)
print "=== credentials=";m.credentials

	m.PrDebug("ssm_AddHttpHandlers end")

End Sub

Sub GetDeviceInfo(userData as Object, e as Object)

	if type(m.mVar.deviceInfo) = "roDeviceInfo" then
		retAA = {
			serial : m.mVar.deviceInfo.GetDeviceUniqueId(),
			model : m.mVar.deviceInfo.GetModel(),
			deviceVersion : m.mVar.deviceInfo.GetVersion(),
			applicationVersion: m.mVar.version
			hdmiIn: m.mVar.deviceInfo.HasFeature("HDMI Input")
			encoder: m.mVar.deviceInfo.HasFeature("video_encoder")
		}
		m.mVar.SendHttpResponse(e, retAA, 200)
	else
		m.mVar.SendHttpResponse(e, "Invalid DeviceInfo", 400)
	end if

End Sub

Sub GetNetworkInfo(userData as Object, e as Object)

	cc = userData.mVar.networkConfig.GetCurrentConfig()
	retAA = {
			hostname : cc.hostname
			ipAddress : cc.ip4_address
			subnetMask : cc.ip4_netmask
			defaultGateway : cc.ip4_gateway
			macAddress : cc.ethernet_mac
			domain : cc.domain
			dhcp : cc.dhcp
			dnsServers : cc.dns_servers
	}
	m.mVar.SendHttpResponse(e, retAA, 200)

End Sub

Sub SetNetworkSettings(userData as Object, e as Object)

	userData.mVar.PrDebug("SetNetworkSettings: start")

	jsonStr$ = e.GetRequestBodyString()
	if jsonStr$ = invalid then
		print "!!! SetNetworkSettings: invalid JSON"
		e.SetResponseBodyString("Bad request")
		e.SendResponse(400)
		return
	end if

	userData.mVar.PrDebug("SetNetworkSettings: jsonStr= "+jsonStr$)
	params = ParseJson(jsonStr$)
	if params = invalid then
		e.SetResponseBodyString("Bad request")
		e.SendResponse(400)
		return
	end if

	nc = CreateObject("roNetworkConfiguration", 0)
	if type(nc) <> "roNetworkConfiguration" then
		e.SetResponseBodyString("Error creating network configuration")
		e.SendResponse(400)
		return
	end if

	success = true

	' Host Name
	if params.hostname <> invalid then
		success = success and nc.SetHostName(params.hostname.trim())
	end if

	' Domain Name
	if params.domainName <> invalid then
		success = success and nc.SetDomain(params.domainName.trim())
	end if

	' DHCP
	if params.dhcp <> invalid and params.dhcp then
		success = success and nc.SetDHCP()
	else
		' Ip Address
		if params.ipAddress <> invalid then
			success = success and nc.SetIP4Address(params.ipAddress.trim())
		end if

		' Subnet Mask
		if params.subnetMask <> invalid then
			success = success and nc.SetIP4Netmask(params.subnetMask.trim())
		end if

		'Default Gateway
		if params.defaultGateway <> invalid then
			success = success and nc.SetIP4Gateway(params.defaultGateway.trim())
		end if

		' DNS Server
		if params.dnsServers <> invalid then
			' Parse the comma-separated list
			regex = CreateObject("roRegEx", ",", "i")
			dnsServers = regex.Split(params.dnsServers)
			if dnsServers.Count() > 0 then
				for each dnsServer in dnsServers
					success = success and nc.AddDNSServer(dnsServer.trim())
				next
			end if
		else if params.defaultGateway <> invalid then
			success = success and nc.AddDNSServer(params.defaultGateway.trim())
		end if 
	end if 

	if success then
		if nc.Apply() then
			e.SetResponseBodyString("OK")
			e.SendResponse(200)
		else
			e.SetResponseBodyString(nc.GetFailureReason())
			e.SendResponse(400)
		end if
	else
		e.SetResponseBodyString(nc.GetFailureReason())
		e.SendResponse(400)
	end if

End Sub

Sub GetFolderContents(userData as Object, e as Object)

	fileList = MatchFiles("\streamCache", "*.*")
	max = fileList.Count()
	sortedFileArray = CreateObject("roArray", max, FALSE)
	' sort files alphabetically
	while fileList.Count() > 0 	
		index = 0
		for i = 1 to fileList.Count()-1
			if LCase(fileList[i]) < LCase(fileList[index]) then
				index = i
			endif
		end for
		sortedFileArray.push(fileList[index])
		fileList.Delete(index)
	end while
	ret$ = FormatJson(sortedFileArray)
	m.mVar.SendHttpResponse(e, ret$, 200)

End Sub

Sub GetStreamingStatus(userData as Object, e as Object)

	mVar = userData.mVar
	retAA = {
		status: mVar.streamingStatus
	}
	if mVar.streamingStatus = "Error" then
		retAA.error = mVar.lastError$
	end if

	mVar.SendHttpResponse(e, retAA, 200)

End Sub

Sub GetCurrentStreamingJob(userData as Object, e as Object)

	mVar = userData.mVar
	mVar.PrDebug("GetCurrentStreamingJob start")
	retAA = {
		status: mVar.streamingStatus
		job: mVar.currentStreamingJob
	}
	if mVar.streamingStatus = "Error" then
		retAA.error = mVar.lastError$
	end if

	m.mVar.SendHttpResponse(e, retAA, 200)
	mVar.PrDebug("GetCurrentStreamingJob end")

End Sub

Sub SetStreamingJob(userData as Object, e as Object)

	mVar = userData.mVar
	mVar.PrDebug("SetStreamingJob start")
	jsonStr$ = e.GetRequestBodyString()
	if jsonStr$ = invalid then
		print "!!! SetStreamingJob: invalid JSON"
		e.SetResponseBodyString("Bad request")
		e.SendResponse(400)
		return
	end if

	mVar.PrDebug("SetStreamingJob: jsonStr= "+jsonStr$)
	params = ParseJson(jsonStr$)
	if params = invalid then
		e.SetResponseBodyString("Bad request")
		e.SendResponse(400)
		return
	end if

	action = LCase(params.action)
	mVar.SaveJob(params)
	mVar.currentStreamingJob = params
	if action = "stop" then
		mVar.StopStreaming()
	else if action = "pause" then
		mVar.PrDebug("pause")
	else if action = "play" then
		mVar.StartStreaming()
	end if

	retAA = {
		status: mVar.streamingStatus
	}
	if mVar.streamingStatus = "Error" then
		retAA.error = mVar.lastError$
	end if
	m.mVar.SendHttpResponse(e, retAA, 200)

End Sub

Sub ssm_StartStreaming()

	m.PrDebug("ssm_StartStreaming start")
	pipelineAA = GetPipelineString(m.currentStreamingJob)
	if pipelineAA.error <> invalid then
		m.streamer = invalid
		m.pipeline$ = ""
		m.streamingStatus = "Error"
		m.lastError$ = pipelineAA.error
		print "!!!--- Stream start failed, error: ";m.lastError$
	else
		m.streamer = CreateObject("roMediaStreamer")
		m.pipeline$ = pipelineAA.pipeline
		m.PrDebug("Starting streaming with pipeline: "+m.pipeline$)
		m.streamer.SetPipeline(m.pipeline$)
		if m.streamer.Start() then
			m.streamingStatus = "Streaming"
			m.ShowLoadingMessage(false)
			m.lastError$ = ""

			' Set up video player to play out HDMI (multicast only)
			if m.currentStreamingJob.destType = "multicast" then
				m.videoPlayer = CreateObject("roVideoPlayer")
				m.rtspStream = CreateObject("roRtspStream", pipelineAA.stream)
				isPlayingHdmi = m.videoPlayer.PlayFile({rtsp:m.rtspStream})
				if isPlayingHdmi then
					m.PrDebug("Playing stream from HDMI out")
				else
					print "=== Error trying to play stream out HDMI"
				end if
			end if
		else
			m.streamingStatus = "Error"
			m.lastError$ = m.streamer.GetFailureReason()
			m.streamer = invalid
			print "---!!!--- Stream start failed, error: ";m.lastError$
		end if
	end if

End Sub

Sub ssm_StopStreaming()

	m.streamingStatus = "Stopped"
	if m.videoPlayer <> invalid then
		m.PrDebug("Stop HDMI video playback")
		m.videoPlayer.Stop()
		m.rtspStream = invalid
		m.videoPlayer = invalid
		m.ShowLoadingMessage(true)
	end if
	if m.streamer <> invalid then
		m.PrDebug("Stop streaming")
		m.streamer.Reset()
	end if

End Sub

Function GetPipelineString(params as Object) as Object

	retAA = { pipeline: invalid, stream: invalid, error: invalid }
	if params <> invalid then
		if LCase(params.source) = "file" then
			if params.file.Len() > 0 then
				filePath = "streamCache/" + params.file
			else
				retAA.error = "No file specified"
				' TODO - check for FileExists
				return retAA
			end if
		end if
		if not LCase(params.protocol) = "udp" and not LCase(params.protocol) = "rtp" then
			retAA.error = "Invalid protocol"
			return retAA
		end if

		useEncoder = params.transcode or LCase(params.source) = "hdmi"
		if useEncoder and (Len(params.encVideoFormat) = 0 or params.encBitrate = 0) then
			retAA.error = "Invalid encoding parameters"
			return retAA
		end if

		if useEncoder then
			if LCase(params.source) = "file" then
				str = "file:///" + filePath
				if params.loop then
					str = str + "?loop"
				end if
				str = str + ", decoder:, "
			else
				str = "hdmi:, "
			end if
			str = str + "encoder:vformat=" + params.encVideoFormat + "&vbitrate=" + StripLeadingSpaces(stri(params.encBitrate))
			str = str + ", " + LCase(params.protocol) + "://"
		else if LCase(params.source) = "file" then
			'filesimple
			str = "filesimple:///" + filePath
			if params.loop then
				str = str + "?loop"
			end if
			str = str + ", " + LCase(params.protocol) + "simple://"
		else
			retAA.error = "Invalid source"
			return retAA
		end if
		retAA.pipeline = str + params.destAddr + ":" + params.destPort + "/?ttl=" + StripLeadingSpaces(stri(params.destTtl))
		retAA.stream = LCase(params.protocol) + "://" + params.destAddr + ":" + params.destPort + "/"
	end if

	return retAA

End Function

' use this to send plugin events to a BrightAuthor Project

Sub ssm_SendPluginEvent(message as string)
 	pluginMessageCmd = CreateObject("roAssociativeArray")
	pluginMessageCmd["EventType"] = "EVENT_PLUGIN_MESSAGE"
	pluginMessageCmd["PluginName"] = "streamingServer"
	pluginMessageCmd["PluginMessage"] = message
	m.msgPort.PostMessage(pluginMessageCmd)
End Sub


Sub ssm_ReadValueFromRegistry(key$ As String)

	if m.storedConfig[key$].type = "D" then 
		valueStr = m.registrySection.Read(key$)
		if len(valueStr) < 1 then
			m.storedConfig[key$].value = m.storedConfig[key$].default 
		else
			m.storedConfig[key$].value = int(val(valueStr))
		end if
	else if m.storedConfig[key$].type = "B" then 
		boolStr = m.registrySection.Read(key$)
		if len(boolStr) < 1 then
			m.storedConfig[key$].value = m.storedConfig[key$].default 
		else if boolStr="true" then
			m.storedConfig[key$].value = true
		else
			m.storedConfig[key$].value = false
		end if
	else if m.storedConfig[key$].type = "S" then 
		strVal = m.registrySection.Read(key$)
		if len(strVal) < 1 then
			m.storedConfig[key$].value = m.storedConfig[key$].default 
		else
			m.storedConfig[key$].value = strVal
		end if
	end if

End Sub

Function ssm_GetValue(key$ As String)

	m.PrDebug("ssm_GetValue  "+key$+" = "+m.storedConfig[key$].value)

	return m.storedConfig[key$].value

End Function

Sub ssm_SaveValue(key$ As String, value)

	m.storedConfig[key$].value = value
	m.PrDebug("ssm_SaveValue: writing "+key$)
	if m.storedConfig[key$].type = "D" then 
		m.registrySection.Write(key$, stri(value).trim())
	else if m.storedConfig[key$].type = "B" then 
		if value then 
			m.registrySection.Write(key$, "true")
		else
			m.registrySection.Write(key$, "false")
		end if
	else if m.storedConfig[key$].type = "S" then 
		m.registrySection.Write(key$, value)
	end if
	m.registrySection.Flush()

End Sub

' Keep track of all registry settings 
Sub ssm_InitStoredConfig()

	m.PrDebug("ssm_InitStoredConfig start")

	m.storedConfig = {}
	m.storedConfig.action = {name: "action", type: "S", value: "stop", default: "stop", reg: true}
	m.storedConfig.codec = {name: "codec", type: "S", value: "H264", default: "H264", reg: true}
	m.storedConfig.destAddr = {name: "destAddr", type: "S", value: "239.194.0.2", default: "239.194.0.2", reg: true}
	m.storedConfig.destPort = {name: "destPort", type: "S", value: "1234", default: "1234", reg: true}
	m.storedConfig.destTtl = {name: "destTtl", type: "D", value: 1, default: 1, reg: true}
	m.storedConfig.destType = {name: "destType", type: "S", value: "multicast", default: "multicast", reg: true}
	m.storedConfig.encBitrate = {name: "encBitrate", type: "D", value: 2000, default: 2000, reg: true}
	m.storedConfig.encVideoFormat = {name: "encVideoFormat", type: "S", value: "720p30", default: "720p30", reg: true}
	m.storedConfig.file = {name: "file", type: "S", value: "", default: "", reg: true}
	m.storedConfig.loop = {name: "loop", type: "B", value: true, default: true, reg: true}
	m.storedConfig.protocol = {name: "protocol", type: "S", value: "udp", default: "udp", reg: true}
	m.storedConfig.source = {name: "source", type: "S", value: "file", default: "file", reg: true}
	m.storedConfig.transcode = {name: "transcode", type: "B", value: false, default: false, reg: true}

	m.PrDebug("ssm_InitStoredConfig before reg")
	' Read the values from the registry
	for each key in m.storedConfig
		m.ReadValueFromRegistry(key)
		m.PrDebug("ssm_InitStoredConfig key="+key)
	next
	
End Sub

Function ssm_GetJob() as Object

	job = {}
	job.action= m.storedConfig["action"].value
	job.codec= m.storedConfig["codec"].value
	job.destAddr= m.storedConfig["destAddr"].value
	job.destPort= m.storedConfig["destPort"].value
	job.destTtl= m.storedConfig["destTtl"].value
	job.destType= m.storedConfig["destType"].value
	job.encBitrate= m.storedConfig["encBitrate"].value
	job.encVideoFormat= m.storedConfig["encVideoFormat"].value
	job.file= m.storedConfig["file"].value
	job.loop= m.storedConfig["loop"].value
	job.protocol= m.storedConfig["protocol"].value
	job.source= m.storedConfig["source"].value
	job.transcode= m.storedConfig["transcode"].value

	return job

End Function


Sub ssm_SaveJob(job as Object)

	m.SaveValue("action", job.action)
	m.SaveValue("codec", job.codec)
	m.SaveValue("destAddr", job.destAddr)
	m.SaveValue("destPort", job.destPort)
	m.SaveValue("destTtl", job.destTtl)
	m.SaveValue("destType", job.destType)
	m.SaveValue("encBitrate", job.encBitrate)
	m.SaveValue("encVideoFormat", job.encVideoFormat)
	m.SaveValue("file", job.file)
	m.SaveValue("loop", job.loop)
	m.SaveValue("protocol", job.protocol)
	m.SaveValue("source", job.source)
	m.SaveValue("transcode", job.transcode)
	
End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub SendHttpResponse(e as Object, retAaOrStr, statusCode as Integer)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	If type(retAaOrStr) = "roAssociativeArray" Then
		ret$ = FormatJson(retAaOrStr)
	Else
		ret$ = retAaOrStr
	End If
	If m.debugMode$ = "1" Then
		e.AddResponseHeader("Access-Control-Allow-Origin", "http://localhost:9090")
	End If
	e.SetResponseBodyString(ret$)
	e.SendResponse(statusCode)
End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Notify(message1 As String, message2 As String, delay As Integer)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	m.PrDebug("Notify message 1: " + message1)
	m.PrDebug("Notify: message 2 " + message2)
	m.PrDebug("Notify: delay: " + str(delay))

	If m.tw = invalid Or m.tw2 = invalid Then
		resX = m.bsp.videoMode.GetResX()
		resY = m.bsp.videoMode.GetResY()

		textParameters = CreateObject("roAssociativeArray")
		textParameters.LineCount = 1
		textParameters.TextMode = 2
		textParameters.Rotation = 0
		textParameters.Alignment = 1
	End If

	If m.tw = invalid Then
		r=CreateObject("roRectangle",0,resY/2-resY/32,resX,resY/32)
		m.tw = CreateObject("roTextWidget", r, 1, 2, textParameters)
	End If

	If m.tw2 = invalid Then
		r2=CreateObject("roRectangle",0,resY/2,resX,resY/32)
		m.tw2=CreateObject("roTextWidget",r2,1,2,textParameters)
	End If

	m.tw.Clear()
	m.tw.PushString(message1)
	m.tw.Show()

	m.tw2.Clear()
	m.tw2.PushString(message2)
	m.tw2.Show()

	If delay > 0 Then
		Sleep(delay)
		m.tw.Hide()
		m.tw2.Hide()
	End If

End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub ShowLoadingMessage(show)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	m.PrDebug("ShowLoadingMessage start")
	msgString$ = ""
	msgString2$ = ""
	ipAddress$ = ""
	If type(m.networkConfig) = "roNetworkConfiguration" Then
		cc = m.networkConfig.GetCurrentConfig()
		If type(cc) = "roAssociativeArray" And cc.ip4_address <> "" Then
			m.PrDebug("ShowLoadingMessage: cc = " + FormatJson(cc))
			ipAddress$ = cc.ip4_address
		End If
		If ipAddress$ <> "" Then
			msgString$ = "Please connect to: http://" + cc.ip4_address + ":8008/"
		Else
			msgString$ = "No network connection"
		End If
	End If
print "=== msgString$ = ";msgString$

	onScreenMessageTime = -1

	If show Then
		m.Notify(msgString$, msgString2$, onScreenMessageTime)
	Else
		If m.tw <> invalid Then
			m.tw.Hide()
		End If
		If m.tw2 <> invalid Then
			m.tw2.Hide()
		End If
	End If
End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub PrDebug(msg$ As String)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	If m.debugMode$ = "1" Then
		print "=== ";msg$
	End If
End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub InitLwsCredentials()
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	m.PrDebug("InitLwsCredentials start")
	if not m.registrySection.Exists("lwsUsername") then
		lwsUsername$ = "admin"
		m.registrySection.Write("lwsUsername", lwsUsername$)
	else
		lwsUsername$ = m.registrySection.Read("lwsUsername")
	end if
	if not m.registrySection.Exists("lwsPassword") then
		lwsPassword$ = "admin"
		m.registrySection.Write("lwsPassword", lwsPassword$)
	else
		lwsPassword$ = m.registrySection.Read("lwsPassword")
	end if

		
	if len(lwsUsername$) > 0 and len(lwsPassword$) > 0 then
		if m.credentials = invalid then
			m.credentials = CreateObject("roAssociativeArray")
		end if
		m.credentials.AddReplace(lwsUsername$, lwsPassword$)
	else
		m.credentials = invalid
	end if
	'''''''''''''''''''' 8< DEBUG '''''''''''''''''''' 
	If m.debugMode$ = "1" Then
		m.credentials = invalid
	End If
	'''''''''''''''''''' >8 DEBUG '''''''''''''''''''' 
	m.PrDebug("InitLwsCredentials end")

End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub GetLwsCredentials(userData as Object, e as Object)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'print "=== GetLwsCredentials start ";m.mVar
	lwsUsername$ = m.mVar.registrySection.Read("lwsUsername")
	lwsPassword$ = m.mVar.registrySection.Read("lwsPassword")
	' We don't return the actual password for security reasons
	dummyPassword$ = ""
	if len(lwsPassword$) > 0 then dummyPassword$ = "*^*^"
	
	retAA = {
		lwsUsername : lwsUsername$
		lwsPassword : dummyPassword$
	}
	m.mVar.SendHttpResponse(e, retAA, 200)

'print "=== GetLwsCredentials end"
End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub SaveLwsCredentials(userData as Object, e As Object)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	userData.mVar.PrDebug("SetLwsCredentials: start")

	jsonStr$ = e.GetRequestBodyString()
	if jsonStr$ = invalid then
		e.SetResponseBodyString("Bad request")
		e.SendResponse(400)
		return
	end if
	args=ParseJson(jsonStr$)
	if args = invalid or args.lwsUsername = invalid or args.lwsPassword = invalid then
		e.SetResponseBodyString("Invalid username or password")
		e.SendResponse(400)
		return
	end if

	lwsUsername$ = args.lwsUsername
	lwsPassword$ = args.lwsPassword

	if len(lwsUsername$) < 1 or len(lwsPassword$) < 1 then
		e.SetResponseBodyString("Invalid username or password")
		e.SendResponse(400)
		return
	end if
		
	userData.mVar.registrySection.Write("lwsUsername", lwsUsername$)
	userData.mVar.registrySection.Write("lwsPassword", lwsPassword$)

	if userData.mVar.credentials = invalid then
		userData.mVar.credentials = CreateObject("roAssociativeArray")
	end if

	userData.mVar.credentials.AddReplace(lwsUsername$, lwsPassword$)

	userData.mVar.SendHttpResponse(e, "ok", 200)

End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub AddUrls(parent$ as String, dir$ as String)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	if parent$ = "" then
		path$ = dir$
	else
		path$ = parent$ + "/" + dir$
	endif
	myFiles = ListDir("/"+path$)
	for each myFile in myFiles
		ext = GetFileExtension(myFile)
		if ext <> invalid then
			contentType$ = GetMimeTypeByExtension(ext)
			if contentType$ <> invalid then
				url$ = "/" +path$+"/"+ myFile
				filenamePath = path$+"/"+myFile 
				m.bsp.sign.localServer.AddGetFromFile({ url_path: url$, filename: filenamePath, content_type: contentType$, passwords: m.credentials })
			endif
		else
			' Assume this is a directory
			m.AddUrls(path$, myFile)
		endif
	next
End Sub
